#include "category.h"

char* get_lowlevel_from_range(const char *context_range)
{
	if (context_range == NULL) return NULL;
	char *rlevel;
	char *range = strdup(context_range);
	char *ch = strchr(range, '-');
	if (ch != NULL) *ch = 0;
	rlevel = strdup(range);
	free(range);
	return rlevel;
}

char* get_highlevel_from_range(const char *context_range)
{
	if (context_range == NULL) return NULL;
	char *rlevel;
	char *range = strdup(context_range);
	char *ch = strchr(range, '-');
	if (ch != NULL) rlevel = strdup(ch + 1);
	else rlevel = NULL;
	free(range);
	return rlevel;
}

void get_sensitivity_and_categoryset_from_level(const char *context_level, char **sensitivity, char **categoryset)
{
	if (context_level == NULL)
	{
		if (categoryset != NULL) *categoryset = NULL;
		if (sensitivity != NULL) *sensitivity = NULL;
		return;
	}

	char *rsensitivity, *rcategoryset;

	char *level = strdup(context_level);
	char *ch = strchr(level, ':');
	if (ch != NULL)
	{
		*ch = 0;
		rcategoryset = strdup(ch + 1);
		if (categoryset != NULL) *categoryset = rcategoryset;
	}
	else if (categoryset != NULL) *categoryset = NULL;

	rsensitivity = strdup(level);
	if (sensitivity != NULL) *sensitivity = rsensitivity;

	free(level);
}

void _add_category_to_array(const char *categoryset, char ***array, int *size)
{
	char *rcategoryset = strdup(categoryset);
	char *ch = strchr(rcategoryset, '.');
	if (ch != NULL)
	{
		*ch = 0;
		int min = atol(rcategoryset + 1);
		int max = atol(ch + 2);
		if (min > max)
		{
			free(rcategoryset);
			return;
		}
		*array = (char**)realloc(*array, ((*size) + (max - min + 1) + 1) * sizeof(char*));
		int i;
		for(i = 0; i < (max - min + 1); i++)
		{
			if (asprintf(&((*array)[(*size) + i]), "c%d", min + i) < 0)
			{
				(*array)[(*size) + i] = NULL;
				free(rcategoryset);
				return;
			}
		}
		(*array)[(*size) + (max - min + 1)] = NULL;
		(*size) += (max - min + 1);
	}
	else
	{
		*array = (char**)realloc(*array, ((*size) + 2) * sizeof(char*));
		(*array)[*size] = strdup(rcategoryset);
		(*array)[(*size) + 1] = NULL;
		(*size)++;
	}
	free(rcategoryset);
}

char** get_category_array_from_categoryset(const char *context_categoryset, int *size)
{
	*size = 0;
	char **result = (char**)calloc(1, sizeof(char*));
	result[0] = NULL;
	if (context_categoryset == NULL || context_categoryset[0] == 0)
	{
		return result;
	}
	char *ch;
	char *categoryset = strdup(context_categoryset);
	char *op = categoryset;

	while((ch = strchr(op, ',')) != NULL)
	{
		*ch = 0;
		_add_category_to_array(op, &result, size);
		op = ch + 1;
	}

	_add_category_to_array(op, &result, size);

	free(categoryset);
	return result;
}
