#include <QApplication>
#include <QTextEdit>

#include <sys/types.h>
#include <sys/wait.h>
//#include <syslog.h>
#include <signal.h>

#include <fcntl.h>
#include <unistd.h>

#include <selinux/context.h>
#include "qtsingleapplication.h"

#include "tray.h"
#include "category.h"

extern const char *__progname;

int WorkApp(int argv, char **args);
int MonitorProc(int argv, char **args);

void Pidfile_create(int pid);
int Pidfile_get();

void SavePolicity_destroy();
void SaveTip_destroy();

void Policity_read();
void Configure_selinux();
char* LvlCatstr();

int asize;
int asize_low;
char *tipLvlCat;

char *seuser = NULL;
char *urange = NULL;
char *ahighsensitivity = NULL;
char *alowsensitivity = NULL;
int ahighsensitivityi=0;
int alowsensitivityi=0;
char *ahighcategoryset = NULL;
char *alowcategoryset = NULL;
char **acategory = NULL;
char **acategory_low = NULL;
char *uhighlevel = NULL;
char *ulowlevel = NULL;
int se_enable;

int main(int argv, char **args)
{

    if ( (argv > 1) && ( !strcmp(args[1], "print") ) ) {
        Configure_selinux();
	if (Pidfile_get() < 0){
	    printf("%s",ulowlevel);
	}
	else {
	    Policity_read();
	    printf("%s",LvlCatstr());
	    return 0;
	}
    }

    if ( (argv == 3) && ( !strcmp(args[1], "color") ) ) {
	Configure_selinux();
        if (Pidfile_get() >= 0) Policity_read();
	if (ahighsensitivityi == 0) return 1;

	const char *context_range;
	char *range = NULL;
	context_t context = context_new(args[2]);
	if (context == NULL) return 2;
	context_range = context_range_get(context);
	if (context_range == NULL || *context_range == 0) return 3;
	range = strdup(context_range);
	if (range == NULL) return 4;
	context_free(context);

	unsigned char rgb[]={0,255,0,200};
	char *level = get_lowlevel_from_range(range);
	if (level == NULL) return 5;
	char *sensitivity;
	get_sensitivity_and_categoryset_from_level(level, &sensitivity, NULL);
	if (sensitivity == NULL || strlen(sensitivity) < 2 || *sensitivity != 's') return 6;
	int sensitivityi = atoi(sensitivity + 1);
	float mult=(float)sensitivityi/(float)(ahighsensitivityi);
	rgb[0]=rgb[1]*mult;
	rgb[1]-=rgb[0];
	printf("%d %d %d\n",rgb[0],rgb[1],rgb[2]);
	return 0;
    }


    int pid = fork();
    if (pid == -1)
    {
	//printf("Error: Start Daemon failed (%s)\n", strerror(errno));
	return -1;
    }
    else if (!pid)
    {
	//umask(0);
	setsid();
	//chdir("/");

	close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);

	return MonitorProc(argv, args);
    }
    else
    {
       return 0;
    }
    //return 0;
}

int MonitorProc(int argv, char **args)
{
    int pid;

    for(;;)
    {
        pid = fork();
	
        if (pid == -1)
	{
	//printf("Error: Start Daemon failed (%s)\n", strerror(errno));
	    return -1;
	}
	else if (!pid)
	{
	    //umask(0);
	    setsid();
	    //chdir("/");
	    close(STDIN_FILENO);
	    close(STDOUT_FILENO);
	    close(STDERR_FILENO);

	    return WorkApp(argv, args);
	}
	else
	{

	    sigset_t sigset;
	    siginfo_t siginfo;

	    sigemptyset(&sigset);
	    sigaddset(&sigset, SIGCHLD);
	    sigaddset(&sigset, SIGTERM);
	    sigprocmask(SIG_BLOCK, &sigset, NULL);
	
	    while ((siginfo.si_signo != SIGCHLD)&&(siginfo.si_signo != SIGTERM))
                sigwaitinfo(&sigset, &siginfo);
	    //wait(&status);
	    //if (status==0) {
	    kill(pid, SIGKILL);
	    SaveTip_destroy();
	    return 0;
        }   
    }
}

int WorkApp(int argv, char **args)
{
    Configure_selinux();
    int status=0;

    QtSingleApplication app(QString("secure_applet_").append(qgetenv("DISPLAY")), argv, args);
    
    if( !app.sendMessage("WAKEUP") )
    {
	Pidfile_create(getpid());
	QTranslator translator;
	translator.load("seapplet_" + QLocale::system().name(), "/usr/share/seapplet/translations");
	app.installTranslator(&translator);
	bool systray_available = QSystemTrayIcon::isSystemTrayAvailable();
	if( !systray_available ) {
	    for( int i=0; i<30 && !systray_available; i++ ) {
		    ::sleep(2);
		    systray_available = QSystemTrayIcon::isSystemTrayAvailable();
	    }
	    if( !systray_available ) {
		QMessageBox::critical(0,QLatin1String(__progname),  QObject::tr("No system tray available."), QMessageBox::Ok, Qt::NoButton);
	    }
	}
	Tray tray(&app,asize,acategory,alowsensitivityi,ahighsensitivityi,se_enable,tipLvlCat);
	status =  app.exec();
    }
    
    free(seuser);
    free(ahighsensitivity);
    free(ahighcategoryset);
    free(uhighlevel);
    free(acategory);
    free(urange);

    return status;
}

void Configure_selinux(){    
    se_enable = is_selinux_mls_enabled();
//    se_enable=1;    
    
    if (se_enable>0){
        if (getseuserbyname(getlogin(), &seuser, &urange))
	{
    	    exit(1);
	}
	//urange=strdup("s4:c1,c3,c6.c8-s5:c1.c8,c10,c12");

//highlevel
	uhighlevel = get_highlevel_from_range(urange);
	if (uhighlevel == NULL) uhighlevel = get_lowlevel_from_range(urange);
	get_sensitivity_and_categoryset_from_level(uhighlevel, &ahighsensitivity, &ahighcategoryset);
	(ahighsensitivity==NULL)?ahighsensitivityi=0:ahighsensitivityi=atoi(ahighsensitivity + 1);
	acategory = get_category_array_from_categoryset(ahighcategoryset, &asize);

//lowlevel
	ulowlevel = get_lowlevel_from_range(urange);
	get_sensitivity_and_categoryset_from_level(ulowlevel, &alowsensitivity, &alowcategoryset);
	(alowsensitivity==NULL)?alowsensitivityi=0:alowsensitivityi=atoi(alowsensitivity + 1);
	acategory_low = get_category_array_from_categoryset(alowcategoryset, &asize_low);

	tipLvlCat = new char [asize+1];
	for (int i=0; i < asize + 1; i++)
	    tipLvlCat[i]=0;

	int j=0;
        tipLvlCat[0]=alowsensitivityi;
	for (int i=0;(i<asize)&&(j<asize_low); i++)
		if (!strcmp(acategory_low[j], acategory[i])){
		    j++;
		    tipLvlCat[i+1]=1;
		}
    }
}

void SaveTip_destroy()
{
    int shmid;
    key_t key;
    int shared_memory_size=asize+1;
    char msg[48];
    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));
    key = ftok(msg,'a');
    if ((shmid = shmget(key, sizeof(char)*shared_memory_size, 0)) < 0) return;
    if (shmctl(shmid, IPC_RMID, NULL) < 0) return;
}

void Policity_read()
{
    int shmid;
    key_t key;
    int shared_memory_size=asize+1;
    char msg[48];
    char *arr;

    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));
    key = ftok(msg,'a');
    if ((shmid = shmget(key, sizeof(char)*shared_memory_size, 0)) < 0) return;
    arr = (char *)shmat(shmid, NULL, 0);
    memcpy( tipLvlCat, arr, sizeof(char)*shared_memory_size);
}

void Pidfile_create(int pid)
{
    char msg[48];
    char dir[48];
    FILE* filepid;

    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));
    sprintf(dir,"%s/secures",getenv("TMPDIR"));
    if (access(dir, F_OK)) system("mkdir -p $TMPDIR/secures");
    //filepid = fopen (msg, O_RDWR|O_CREAT,0640);
    filepid = fopen(msg,"wb+");
    fprintf(filepid,"%d",pid);
    fclose(filepid);
}

int Pidfile_get()
{
    char msg[48];
    int pid=-1;
    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));

    FILE* filepid;
    filepid = fopen(msg,"r");
    if (!filepid) return -1;
    if (!fscanf(filepid, "%d",&pid)) return -1;
    fclose(filepid);
    if (kill(pid, 0)) return -1;
    return pid;
}

char* LvlCatstr()
{
    int i=0,j=0;
    bool k=true;
    int numcategory[asize];
    
    for (int i=0; i<asize;i++){
	numcategory[i]=atoi(acategory[i] + 1);
    }
    
    QString tip = QString("s%1").arg((int)tipLvlCat[0]);
    
    while (i<=asize){
        if (tipLvlCat[i+1]==1) 
	{
	    if (k){
	        tip.append(":"); k=false;
	    }
	    else tip.append(",");
	    tip.append(QString("c%1").arg(numcategory[i]));
	    j=i+1;
	    while ((tipLvlCat[j+1]==1)&&(j<asize)&&(numcategory[j]-numcategory[j-1]==1)) j++;
	    if (j>i+2) {
	        tip.append(QString(".c%1").arg(numcategory[j-1]));
		i=j-1;
	    }
	    else if (j==i+2) {
	        tip.append(QString(",c%1").arg(numcategory[j-1]));
		i=j-1;
	    }
	}
	i++;
    }
    QByteArray tmp=tip.toUtf8();
    char *data = tmp.data();
    return data;
}
