#include <QApplication>
#include <QTextEdit>
#include <QTranslator>
#include <QMenu>
#include <qapplication.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpushbutton.h>
#include <qcolor.h>
#include <QPainter>
#include <QPalette>
#include <QBitmap>
#include <QtGui/QPaintEvent>
#include <stdio.h>
#include <selinux/selinux.h>

#include "tray.h"


static char* sensitivity_raw_to_trans(int sensitivity)
{
	char *ret = NULL;
	char *btrans;
	char braw[64];
	snprintf(braw, 64, ":::s%d", sensitivity);
	if (selinux_raw_to_trans_context(braw, &btrans) == 0)
	{
		ret = strdup(btrans + 3);
		freecon(btrans);
	}
	return ret;
}

static char* category_raw_to_trans(int category)
{
	char *ret = NULL;
	char *btrans;
	char braw[64];
	snprintf(braw, 64, ":::c%d", category);
	if (selinux_raw_to_trans_context(braw, &btrans) == 0)
	{
		ret = strdup(btrans + 3);
		freecon(btrans);
	}
	return ret;
}

Tray::Tray(QObject *parent,int max_category_,char **acategory, int lowlvl, int highlvl, int se_enable, char *tipLvlCat_):
	QSystemTrayIcon(QIcon::fromTheme("altlinux", QIcon("contact-new")), parent)
{
    se_enable_=se_enable;
    numcategory = new int [max_category_];
    
    if (se_enable_>0) {
        for (int i=0; i<max_category_;i++){
	    char *trans = category_raw_to_trans(atoi(acategory[i] + 1));
	    if (trans != NULL)
	    {
		category.append(QString("%1").arg(QString::fromUtf8(trans)));
		free(trans);
	    }
	    else category.append(tr("category %1").arg(acategory[i]));
	    numcategory[i]=atoi(acategory[i] + 1);
        }
	//category.append("category 2");

	//max_category=category.size();
	max_category=max_category_;

	//level.append(tr("free"));
	//level.append(tr("secret"));
	//level.append(tr("top secret"));
	max_lvl=highlvl;
	min_lvl=lowlvl;
	for (int i=min_lvl;i<=max_lvl;i++) level.append(tr("top secret %1").arg(i));

	tipLvlCat = new char [max_category+1];;
	for (int i=0;i<max_category+1;i++) tipLvlCat[i]=tipLvlCat_[i];

	connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(onActivateSysTray(QSystemTrayIcon::ActivationReason)));
	connect(this, SIGNAL(messageClicked()), this, SLOT(onClickTrayMessage()));

	menu_ = new QMenu();
	menu_secur  = new QMenu();
	menu_categ  = new QMenu();
	menu_secur = menu_->addMenu(tr("&Level"));
	menu_categ = menu_->addMenu(tr("&Category"));

	for (int i=min_lvl;i<=max_lvl;i++){
	    QAction *level_action = new QAction(this);
	    connect(level_action, SIGNAL(triggered()), this, SLOT(LevelSecure()));
	    menu_secur->addAction(level_action);
	    //QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(category[i]));
	    QString text;
	    char *trans = sensitivity_raw_to_trans(i);
	    if (trans != NULL)
	    {
		text = QString("%1").arg(QString::fromUtf8(trans));
		free(trans);
	    }
	    else text = tr("level s%1").arg(i);
	    level_action->setText(text);
	    level_action->setData(i);
        }

        menu_->addSeparator();

	for (int i=0; i<max_category;i++)
        {
	    QAction *category_action = new QAction(this);
	    category_action->setCheckable(true);
	    connect(category_action, SIGNAL(triggered()), this, SLOT(Category()));
	    QString text(category[i]);
	    category_action->setText(text);
	    category_action->setData(i);
	    category_action->setChecked((bool)tipLvlCat[i+1]);
	    menu_categ->addAction(category_action);
	}

	LevelIcon(tipLvlCat[0]);
	setContextMenu(menu_);
    }
    else {
        unsigned char rgb[]={0,0,0,220};
	pixTrayIcon(rgb);
	setToolTip("SeLinux disable");
    }
}

Tray::~Tray()
{
    SaveTip_destroy();
}

void Tray::Destroy()
{
    SaveTip_destroy();
}

void Tray::pixTrayIcon(unsigned char rgb[4])
{
    //QImage image(geometry().size(), QImage::Format_ARGB32);
    QImage image(22,22, QImage::Format_ARGB32);
    image.fill(0);
    QPainter painter(&image);
    painter.setRenderHints(QPainter::Antialiasing);
    painter.setBrush(QColor(rgb[0], rgb[1], rgb[2], rgb[3]));
    painter.setPen(QApplication::palette().color(QPalette::Highlight));
    static const QPointF points[7] = {
	QPointF(1,1),
	QPointF(20,1),
	QPointF(20,15),
	QPointF(11,20),
	QPointF(10,20),
	QPointF(1,15),
	QPointF(1,1)
    };
    painter.drawPolygon(points,7);
    //painter.drawEllipse(0, 0, 21, 21);
    
    if (se_enable_>0){
        QString text = QString("%1").arg((int)tipLvlCat[0]);
	//QFont font("Arial");
	//font.setPointSize(geometry().size().height()/(text.size()));
	//painter.setFont(font);
	painter.setBrush(Qt::black);
	painter.setPen(Qt::black);
	painter.drawText(image.rect(), Qt::AlignCenter, text);
    }
    painter.end();

    QPixmap pix = QPixmap::fromImage(image);

    setIcon(pix); 
    setVisible(true);
}

void Tray::tipTrayIcon()
{
    int i=0,j=0;
    bool k=true;
    QString tip;
    char *trans = sensitivity_raw_to_trans((int)tipLvlCat[0]);
    if (trans != NULL)
    {
	tip = tr("Level:\t\t%1").arg(QString::fromUtf8(trans));
	free(trans);
    }
    else tip = tr("Level:\t\t%1").arg(level[(int)tipLvlCat[0]]);

    while (i<=max_category){
        if (tipLvlCat[i+1]==1) 
	{
	    if (k){
	        tip.append(tr("\nCategory:\t")); k=false;
	    }
	    else tip.append(",");
	    trans = category_raw_to_trans(numcategory[i]);
	    if (trans != NULL)
	    {
		tip.append(QString("%1").arg(QString::fromUtf8(trans)));
		free(trans);
	    }
	    else tip.append(QString("%1").arg(numcategory[i]));
	    j=i+1;
	    while ((tipLvlCat[j+1]==1)&&(j<max_category)&&(numcategory[j]-numcategory[j-1]==1)) j++;
	    if (j>i+2) {
		trans = category_raw_to_trans(numcategory[j-1]);
		if (trans != NULL)
		{
		    tip.append(QString("-%1").arg(QString::fromUtf8(trans)));
		    free(trans);
		}
		else tip.append(QString("-%1").arg(numcategory[j-1]));
		i=j-1;
	    }
	    else if (j==i+2) {
		trans = category_raw_to_trans(numcategory[j-1]);
		if (trans != NULL)
		{
		    tip.append(QString(",%1").arg(QString::fromUtf8(trans)));
		    free(trans);
		}
		else tip.append(QString(",%1").arg(numcategory[j-1]));
		i=j-1;
	    }
	}
	i++;
    }
    //QByteArray tmp=tip.toUtf8();
    //char *data = tmp.data();

    setToolTip(tip);
    SaveTip(); 
}

void Tray::onActivateSysTray(QSystemTrayIcon::ActivationReason)
{
}

void Tray::onClickTrayMessage()
{
}

void Tray::LevelSecure()
{   
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
        LevelIcon(action->data().toInt());
}

void Tray::Category()
{
    QAction *action = qobject_cast<QAction *>(sender());
    if (action)
        CategoryTip(action->data().toInt());
}

void Tray::LevelIcon(const int param)
{
    unsigned char rgb[]={0,255,0,200};    
    float mult=(float)param/(float)(max_lvl);
    tipLvlCat[0]=param;

    rgb[0]=rgb[1]*mult;
    rgb[1]-=rgb[0];
    pixTrayIcon(rgb);
    tipTrayIcon();
}

void Tray::CategoryTip(const int param)
{
    if (tipLvlCat[param+1]==0) tipLvlCat[param+1]=1;
    else tipLvlCat[param+1]=0;
    tipTrayIcon();
}

void Tray::SaveTip()
{
    int shmid;
    char msg[48];
    char *arr;
    key_t key;
    int shared_memory_size=max_category+1;
    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));
    key = ftok(msg,'a');

    if ((shmid = shmget(key, sizeof(char)*shared_memory_size, IPC_CREAT | IPC_EXCL | 0640)) < 0)
    {
        if (errno != EEXIST) return;
        if ((shmid = shmget(key, sizeof(char)*shared_memory_size, 0)) < 0) return;
    }

    arr = (char *)shmat(shmid, NULL, 0);
    memcpy( arr, tipLvlCat, sizeof(char)*shared_memory_size);

    if (shmdt(arr) < 0) return;
}

void Tray::SaveTip_destroy()
{
    int shmid;
    key_t key;
    int shared_memory_size=max_category+1;
    char msg[48];
    sprintf(msg,"%s/secures/se_app%s",getenv("TMPDIR"),getenv("DISPLAY"));
    key = ftok(msg,'a');

    if ((shmid = shmget(key, sizeof(char)*shared_memory_size, 0)) < 0) return;

    if (shmctl(shmid, IPC_RMID, NULL) < 0) return;
}
